{% extends 'base/layout.html' %}
{% load buttons %}
{% load helpers %}
{% load render_table from django_tables2 %}

{% block title %}DNS Sync Jobs{% endblock %}

{% block controls %}
  <div class="controls">
    <div class="control-group">
      <a href="{% url 'plugins:netbox_powerdns_sync:sync_schedule' %}" type="button" class="btn btn-sm btn-primary" title="Schedule full zone sync">
        <i class="mdi mdi-clock-outline" aria-hidden="true"></i> Schedule
      </a>
    </div>
  </div>
{% endblock controls %}

{% block tabs %}
  <ul class="nav nav-tabs px-3">
    <li class="nav-item" role="presentation">
      <a class="nav-link active" role="tab">Sync Jobs</a>
    </li>
  </ul>
{% endblock tabs %}

{% block content %}
  <div class="tab-content">
    <div class="tab-pane show active" id="object-list" role="tabpanel" aria-labelledby="object-list-tab">
      <div class="card">
        <div class="card-body table-responsive">
          {% render_table table 'inc/table.html' %}
          {% include 'inc/paginator.html' with paginator=table.paginator page=table.page %}
        </div>
      </div>
    </div>
  </div>
{% endblock %}
